//============================================================================//
//============================================================================//

#include "phydrv.h"

#include <map>
#include <string>

#include "bzfsAPI.h"

#include "bzfs.h"
//#include "BzMaterial.h"
#include "MeshFace.h"
#include "MeshObstacle.h"
#include "ObstacleMgr.h"
#include "PhysicsDriver.h"


//============================================================================//
//============================================================================//

PhydrvHandler::PhydrvHandler()
{
}


PhydrvHandler::~PhydrvHandler()
{
}


//============================================================================//

void PhydrvHandler::insertEvents()
{
  bz_registerEvent(bz_ePlayerJoinEvent,        this);
  bz_registerEvent(bz_ePlayerPartEvent,        this);
  bz_registerEvent(bz_ePlayerDieEvent,         this);
  bz_registerEvent(bz_ePlayerSpawnEvent,       this);
  bz_registerEvent(bz_ePlayerUpdateEvent,      this);
  bz_registerEvent(bz_eGetPlayerSpawnPosEvent, this);
}


void PhydrvHandler::removeEvents()
{
  bz_removeEvent(bz_ePlayerJoinEvent,        this);
  bz_removeEvent(bz_ePlayerPartEvent,        this);
  bz_removeEvent(bz_ePlayerDieEvent,         this);
  bz_removeEvent(bz_ePlayerSpawnEvent,       this);
  bz_removeEvent(bz_ePlayerUpdateEvent,      this);
  bz_removeEvent(bz_eGetPlayerSpawnPosEvent, this);
}


//============================================================================//

int PhydrvHandler::getPhydrv(int playerID) const
{
  std::map<int, int>::const_iterator it = players.find(playerID);
  if (it == players.end()) {
    return -1;
  }
  return it->second;
}


void PhydrvHandler::setPhydrv(int playerID, int phydrv)
{
  const int oldPhydrv = getPhydrv(playerID);
  players[playerID] = phydrv;
  if (phydrv != oldPhydrv) {
    phydrvChange(playerID, phydrv, oldPhydrv);
  }
}


//============================================================================//

void PhydrvHandler::process(bz_EventData* ed)
{
  switch (ed->eventType) {
    case bz_ePlayerUpdateEvent: {
      bz_PlayerUpdateEventData* e = (bz_PlayerUpdateEventData*) ed;
      setPhydrv(e->playerID, e->phydrv);
      break;
    }
    case bz_ePlayerJoinEvent: {
      bz_PlayerJoinPartEventData* e = (bz_PlayerJoinPartEventData*) ed;
      setPhydrv(e->playerID, -1);
      break;
    }
    case bz_ePlayerPartEvent: {
      bz_PlayerJoinPartEventData* e = (bz_PlayerJoinPartEventData*) ed;
      setPhydrv(e->playerID, -1);
      break;
    }
    case bz_ePlayerDieEvent: {
      bz_PlayerDieEventData* e = (bz_PlayerDieEventData*) ed;
      setPhydrv(e->playerID, -1);
      break;
    }
    case bz_ePlayerSpawnEvent: {
      bz_PlayerSpawnEventData* e = (bz_PlayerSpawnEventData*) ed;
      setPhydrv(e->playerID, -1);
      break;
    }
    case bz_eGetPlayerSpawnPosEvent: {
      bz_GetPlayerSpawnPosEventData* e = (bz_GetPlayerSpawnPosEventData*) ed;
      setPhydrv(e->playerID, -1);
      break;
    }
    default: {
      break;
    }
  }
}


//============================================================================//
//============================================================================//

static PhydrvHandler::IdMap   idMap;
static PhydrvHandler::NameMap nameMap;


static bool makeMaps()
{
  if (!world) {
    return false;
  }

  if (!idMap.empty()) {
    return true;
  }

  for (int id = 0; /* no-op */; id++) {
    const PhysicsDriver* phydrv = PHYDRVMGR.getDriver(id);
    if (!phydrv) {
      break;
    }
    const std::string& name = phydrv->getName();
    idMap[id] = name;
    nameMap[name] = id;
  }

  return true;
}


const PhydrvHandler::IdMap* PhydrvHandler::getIdMap()
{
  if (!makeMaps()) {
    return NULL;
  }
  return &idMap;
}


const PhydrvHandler::NameMap* PhydrvHandler::getNameMap()
{
  if (!makeMaps()) {
    return NULL;
  }
  return &nameMap;
}


//============================================================================//
//============================================================================//
